<?php

class AitStripe
{

	private static $instance = null;

	public $options;
	public $payment;


	protected function __construct() {}

	public static function getInstance()
	{
		if (null == self::$instance) {
			self::$instance = new self;
			// construct
			self::$instance->setOptions();
			// handle actions
			add_action('wp_ajax_ait_stripe_checkout', array(__CLASS__, 'handleAjaxCheckout'));
			add_action('wp_ajax_nopriv_ait_stripe_checkout', array(__CLASS__, 'handleAjaxCheckout'));
		}
		return self::$instance;
	}

	private function setOptions()
	{
		if (!function_exists('aitOptions')) {
			throw new AitStripeException("Plugin is compatible only with AIT framework 2.0");
		}
		$options = aitOptions()->get('theme');
		if (isset($options->stripe)) {
			$this->options = $options->stripe;
		}
	}

	private function setSecretKey()
	{
		if (empty($this->options)) {
			throw new AitStripeException("Missing theme options");
		}
		if (empty($this->options->live)) {
			Stripe\Stripe::setApiKey($this->options->testSecretKey);
		} else {
			Stripe\Stripe::setApiKey($this->options->liveSecretKey);
		}
	}

	private function getPublishableKey()
	{
		if (empty($this->options)) {
			throw new AitStripeException("Missing theme options");
		}
		if (empty($this->options->live)) {
			return $this->options->testPublishableKey;
		} else {
			return $this->options->livePublishableKey;
		}
	}

	public function requestPayment($data, $name, $description, $amount, $currency = 'USD')
	{
		$this->payment = array(
			'name' => $name,
			'description' => $description,
			'amount' => $amount,
			'currency' => $currency,
			'data' => $data
		);
		add_action('wp_footer', array(__CLASS__, 'generateCheckout'));
		add_action('admin_footer', array(__CLASS__, 'generateCheckout'));
	}

	public function generateCheckout()
	{
		$obj = self::getInstance();
		$paymentJson = json_encode($obj->payment);
		$publishableKey = $obj->getPublishableKey();
		$ajaxUrl = admin_url('admin-ajax.php');
		$ajaxNonce = wp_create_nonce('ait-stripe-ajax-checkout');
		$redirectSuccess = !empty($obj->options->successPage) ? get_permalink($obj->options->successPage)."?ait-notification=user-registration-success" : home_url("/")."?ait-notification=user-registration-success";
		$redirectError = !empty($obj->options->errorPage) ? get_permalink($obj->options->errorPage) : home_url();

		echo <<<BUTTON
		<style>
		.stripe-button-wrap {
			position: fixed;
			top: 50%;
			left: 50%;
			-moz-transform: translate(-50%, -50%);
			-webkit-transform: translate(-50%, -50%);
			-ms-transform: translate(-50%, -50%);
			transform: translate(-50%, -50%);
			width: -webkit-calc(100% - 40px);
			width: calc(100% - 40px);
			max-width: 590px;
			background: #f5f5f7;
			border-radius: 5px;
			padding: 20px;
			z-index: 99999;
			box-sizing: border-box;
			box-shadow: 0 0 5px rgba(0,0,0,0.2), 0 0 0 1px rgba(0,0,0,0.08);
		}

		.stripe-button-wrap svg {
			position: absolute;
			left: 20px;
			top: -35px;
			color: #fff;
			opacity: 0.5;
		}

		.stripe-button-wrap svg {
			fill: currentColor;
		}

		.stripe-button-to-checkout {
			font-family: Roboto, -apple-system, BlinkMacSystemFont, "Helvetica Neue", "Segoe UI", "Oxygen", "Ubuntu", "Cantarell", "Open Sans", sans-serif;
			font-size: 24px;
			font-weight: bold;
			line-height: 1;
			background: #54b0ed;
			color: #fff;
			border-radius: 5px;
			padding: 30px 40px;
			text-align: center;
			box-shadow: 0 1px 0 rgba(0,0,0,0.15), 0 -1px 0 rgba(0,0,0,0.17) inset, 0 -50px 50px -20px #4398e3 inset;
			cursor: pointer;
		}

		.stripe-button-overlay {
			position: fixed;
			top: 0px;
			left: 0px;
			right: 0px;
			bottom: 0px;
			background: rgba(0,0,0,.6);
			z-index: 99999;
		}
		</style>
		<div class="stripe-button-overlay"></div>
		<div class="stripe-button-wrap">
			<svg height="25" width="62"><title>Stripe</title><path d="M5 10.1c0-.6.6-.9 1.4-.9 1.2 0 2.8.4 4 1.1V6.5c-1.3-.5-2.7-.8-4-.8C3.2 5.7 1 7.4 1 10.3c0 4.4 6 3.6 6 5.6 0 .7-.6 1-1.5 1-1.3 0-3-.6-4.3-1.3v3.8c1.5.6 2.9.9 4.3.9 3.3 0 5.5-1.6 5.5-4.5.1-4.8-6-3.9-6-5.7zM29.9 20h4V6h-4v14zM16.3 2.7l-3.9.8v12.6c0 2.4 1.8 4.1 4.1 4.1 1.3 0 2.3-.2 2.8-.5v-3.2c-.5.2-3 .9-3-1.4V9.4h3V6h-3V2.7zm8.4 4.5L24.6 6H21v14h4v-9.5c1-1.2 2.7-1 3.2-.8V6c-.5-.2-2.5-.5-3.5 1.2zm5.2-2.3l4-.8V.8l-4 .8v3.3zM61.1 13c0-4.1-2-7.3-5.8-7.3s-6.1 3.2-6.1 7.3c0 4.8 2.7 7.2 6.6 7.2 1.9 0 3.3-.4 4.4-1.1V16c-1.1.6-2.3.9-3.9.9s-2.9-.6-3.1-2.5H61c.1-.2.1-1 .1-1.4zm-7.9-1.5c0-1.8 1.1-2.5 2.1-2.5s2 .7 2 2.5h-4.1zM42.7 5.7c-1.6 0-2.5.7-3.1 1.3l-.1-1h-3.6v18.5l4-.7v-4.5c.6.4 1.4 1 2.8 1 2.9 0 5.5-2.3 5.5-7.4-.1-4.6-2.7-7.2-5.5-7.2zm-1 11c-.9 0-1.5-.3-1.9-.8V10c.4-.5 1-.8 1.9-.8 1.5 0 2.5 1.6 2.5 3.7 0 2.2-1 3.8-2.5 3.8z"></path></svg>
			<div class="stripe-button-to-checkout">Proceed to checkout</div>
		</div>
		<script src="https://checkout.stripe.com/checkout.js"></script>
		<script>
			var aitStripe = {
				api: StripeCheckout.configure({
					key: '{$publishableKey}',
					token: function(token) {
						var data = {
							action: 'ait_stripe_checkout',
							nonce: '{$ajaxNonce}',
							token: token.id,
							payment: aitStripe.payment
						};
						jQuery.post('{$ajaxUrl}', data, function(data) {
							if(data.redirect) {
								window.location = data.redirect;
							}
						});
					},
					closed: function(){
						setTimeout(function(){
							if(jQuery('body').find('iframe.stripe_checkout_app').length == 0){
								window.location.href = "{$redirectSuccess}";
							}
						}, 500);
					}
				}),
				payment: null,
				requestPayment: function(params) {
					this.payment = params;
					// Zero-decimal currencies
					// https://support.stripe.com/questions/which-zero-decimal-currencies-does-stripe-support
					if (params.currency == 'BIF' || params.currency == 'CLP' || params.currency == 'DJF' || params.currency == 'GNF' || params.currency == 'JPY' || params.currency == 'KMF' || params.currency == 'KRW' || params.currency == 'MGA' || params.currency == 'PYG' || params.currency == 'RWF' || params.currency == 'VND' || params.currency == 'VUV' || params.currency == 'XAF' || params.currency == 'XOF' || params.currency == 'XPF') {
						this.payment.amount = params.amount;
					} else {
						// in cents
						this.payment.amount = params.amount * 100;
					}
					this.api.open(params);
				}
			};
			jQuery(window).on('popstate', function() {
				aitStripe.api.close();
			});
			jQuery('.stripe-button-overlay').on('click', function() {
				if (typeof ait.home.url !== "undefined") {
					var redirect = 
					window.location.href = ait.home.url + "?ait-notification=user-registration-success";
				}
			});
			jQuery('.stripe-button-to-checkout').on('click', function() {
				jQuery('.stripe-button-wrap, .stripe-button-overlay').hide();
				aitStripe.requestPayment({$paymentJson});
			});
		</script>
BUTTON;
	}

	public function handleAjaxCheckout()
	{
		check_ajax_referer('ait-stripe-ajax-checkout', 'nonce');
		$result = array(
			'success' => false
		);
		try {
			$obj = AitStripe::getInstance();
			$obj->setSecretKey();
			$params = array('currency' => 'usd');
			// filter
			foreach ($_POST['payment'] as $key => $value) {
				if (in_array($key, array('description', 'amount', 'currency'))) {
					$params[$key] = $value;
				}
			}
			$params['source'] = $_POST['token'];
			$charge = Stripe\Charge::create($params);
			if ($charge->status == 'succeeded') {
				$payment = (object) $_POST['payment'];
				do_action('ait-stripe-payment-success', $payment, $charge);
				$result['success'] = true;
				if (!empty($obj->options->successPage)) {
					$result['redirect'] = get_permalink($obj->options->successPage);
				}
			} else {
				if (!empty($obj->options->errorPage)) {
					$result['redirect'] = get_permalink($obj->options->errorPage);
				}
			}
		} catch (Exception $e) {
			AitStripe::error($e);
			do_action('ait-stripe-payment-error', $e->getMessage());
			if (!empty($obj->options->errorPage)) {
				$result['redirect'] = get_permalink($obj->options->errorPage);
			}
		}
		wp_send_json($result);
	}

	public static function log($message, $title = '', $fileName = 'info.log')
	{
		if ($message instanceof Exception) {
			$message = $message->getMessage();
			$title = get_class($message);
		} else {
			$message = print_r($message, true);
			$title = (!empty($title)) ? " - " . $title : "";
		}
		$message = date("Y-m-d H:i:s") . $title . "\n\n" . $message . "\n\n";
		// save to uploads
		$path = wp_upload_dir();
		if (is_writable($path['basedir'])) {
			$file = $path['basedir']."/ait/stripe/";
			if (wp_mkdir_p($file)) {
				$file .= $fileName;
				error_log($message, 3, $file);
			}
		}
	}

	public static function error($message)
	{
		self::log($message, '', 'error.log');
	}

}

class AitStripeException extends Exception {}